use serde::{Deserialize, Serialize};
use std::path::PathBuf;

#[derive(Serialize, Deserialize)]
pub struct ImagePosition {
    pub x: u16,
    pub y: u16,
}

impl ImagePosition {
    pub fn new(x: u16, y: u16) -> ImagePosition {
        ImagePosition{x, y}
    }
}

impl std::default::Default for ImagePosition {
    fn default() -> Self {
        ImagePosition::new(0, 0)
    }
}

#[derive(Serialize, Deserialize)]
pub struct ImageSize {
    pub width:  u16,
    pub height: u16,
}

impl ImageSize {
    pub fn new(width: u16, height: u16) -> ImageSize {
        ImageSize{width, height}
    }
}

impl std::default::Default for ImageSize {
    fn default() -> Self {
        ImageSize::new(0, 0)
    }
}

#[derive(Serialize, Deserialize)]
pub struct PaletteRect {
    pub pos:  ImagePosition,
    pub size: ImageSize,
}

impl PaletteRect {
    pub fn new(x: u16, y: u16, width: u16, height: u16) -> PaletteRect {
        PaletteRect{pos: ImagePosition{x, y}, size: ImageSize{width, height}}
    }
}

impl std::default::Default for PaletteRect {
    fn default() -> Self {
        PaletteRect{pos: ImagePosition::default(), size: ImageSize::default()}
    }
}

#[derive(Serialize, Deserialize)]
pub struct Job {
    name:         String,
    file_path:    PathBuf,
    image_pos:    ImagePosition,
    palette_rect: PaletteRect,
}

impl Job {
    pub fn new(name: String, file_path: PathBuf, image_pos: (u16, u16)) -> Job {
        Job{name, file_path, image_pos: ImagePosition{x: image_pos.0, y: image_pos.1}, palette_rect: PaletteRect::default()}
    }

    pub fn add_palette(&mut self, palette_rect: PaletteRect) -> &mut Self {
        self.palette_rect = palette_rect;
        self
    }
}

#[derive(Serialize, Deserialize)]
pub struct Project {
    jobs: Vec<Job>
}

impl Project {
    pub fn new(jobs: Vec<Job>) -> Project {
        Project{jobs}
    }

    pub fn push(&mut self, job: Job) {
        self.jobs.push(job);
    }

    pub fn len(&self) -> usize {
        self.jobs.len()
    }
}